@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -57,6 +57,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /* define when system provides typedef for byte */
 #undef HAVE_U32_TYPEDEF
 
+/* ---------------- istat.c support ----------------
+   (added by AC_DEFINE in autoconf 2.53)
+#undef HAVE_LONG_LONG
+#undef HAVE_UNSIGNED_LONG_LONG
+*/
+
 /* define when system provides stdint.h header */
 #undef HAVE_STDINT_H
 
