@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -122,32 +122,46 @@ inline static int do_checksum(integrit_t *it, fileinfo *inf)
     dbinfo	*dbinf	 = &inf->dbinf;
     struct utimbuf	utb;	/* for resetting access time */
 
-    if ( (fd = open(inf->path, O_RDONLY)) == -1) {
-      warn(__FUNCTION__, "Warning: could not open file (%s) for reading: %s",
-	   inf->path, strerror(errno));
-      return 0;			/* didn't do checksum */
-    }
-    if (fstat(fd, &fd_sb) == -1)
-      die(__FUNCTION__, "Error: could not fstat file (%s): %s\n",
-	  inf->path, strerror(errno));
-
-    /* check that the file we've opened is the same one we did lstat on */
-    if (fd_sb.st_ino != inf->dbinf.stat.st_ino) {
-      warn(__FUNCTION__, "Warning: file (%s) changed before we opened it\n",
-	   inf->path);
-      return 0;			/* didn't do checksum */
-    }
-
     sha1_init(&context);
 
-    /* if read fails, n gets -1 */
-    while ( (n = read(fd, buf, BUFSIZ)) > 0)
-      sha1_write(&context, buf, n);
-    close(fd);
+    /*
+     * For a symlink, read the name in the symlink as if it were "contents".
+     * Note that we can't make this check on the open fd's stat buffer,
+     * because fstat() works on the underlying file, not the symlink.
+     */
+    if (S_ISLNK(inf->dbinf.stat.st_mode)) {
+      int ret;
+      ret = readlink(inf->path, buf, BUFSIZ);
+      if (ret < 0)
+	die(__FUNCTION__, "Error: could not readlink symlink (%s): %s\n",
+	    inf->path, strerror(errno));
+      sha1_write(&context, (uint8_t*) buf, ret);
+    } else {
+      if ( (fd = open(inf->path, O_RDONLY)) == -1) {
+        warn(__FUNCTION__, "Warning: could not open file (%s) for reading: %s",
+	     inf->path, strerror(errno));
+        return 0;			/* didn't do checksum */
+      }
+      if (fstat(fd, &fd_sb) == -1)
+        die(__FUNCTION__, "Error: could not fstat file (%s): %s\n",
+	    inf->path, strerror(errno));
 
-    if (n == -1)		/* read error */
-      warn(__FUNCTION__, "Warning: read from file (%s) failed: %s",
-	   inf->path, strerror(errno));
+      /* check that the file we've opened is the same one we did lstat on */
+      if (fd_sb.st_ino != inf->dbinf.stat.st_ino) {
+        warn(__FUNCTION__, "Warning: file (%s) changed before we opened it\n",
+	     inf->path);
+        return 0;			/* didn't do checksum */
+      }
+      /* if read fails, n gets -1 */
+      while ( (n = read(fd, buf, BUFSIZ)) > 0)
+        sha1_write(&context, (uint8_t*) buf, n);
+
+      if (n == -1)		/* read error */
+        warn(__FUNCTION__, "Warning: read from file (%s) failed: %s",
+	     inf->path, strerror(errno));
+
+      close(fd);
+    }
 
     sha1_final(&context);
     /* It is faster but slightly dirtier to use context.buf directly.
@@ -217,10 +231,10 @@ static void show_diff_xml_octal(FILE *out, char T, const char *type,
 {
     xml_start_print(out, type);
     XML_START_PRINT(out, "old");
-    fprintf(out, "%lo", old);
+    fprintf(out, "0%lo", old);
     XML_END_PRINT(out, "old");
     XML_START_PRINT(out, "new");
-    fprintf(out, "%lo", new);
+    fprintf(out, "0%lo", new);
     XML_END_PRINT(out, "new");
     xml_end_print(out, type);
 }
@@ -228,23 +242,27 @@ static void show_diff_xml_octal(FILE *out, char T, const char *type,
 static void show_diff_octal(FILE *out, char T, const char *type,
 				  unsigned long old, unsigned long new)
 {
-    fprintf(out, "%c(%lo:%lo) ", T, old, new);
+    fprintf(out, "%c(0%lo:0%lo) ", T, old, new);
 }
 
 static void show_diff_perms(FILE *out, mode_t old, mode_t new)
 {
     fputs("p(", out);
-    if (S_ISLNK(old)) /* for a symlink ... */
-      fputs("sym", out); /* display "sym" instead of "777" */
-    else
-      fprintf(out, "%o", (unsigned int) old & PERM_MASK);
+    fprintf(out, "0%o", (unsigned int) old);
 
     putc(':', out);
 
-    if (S_ISLNK(new))
-      fputs("sym) ", out);
-    else
-      fprintf(out, "%o) ", (unsigned int) new & PERM_MASK);
+    fprintf(out, "0%o) ", (unsigned int) new);
+}
+
+static void show_diff_type(FILE *out, mode_t old, mode_t new)
+{
+    fputs("t(", out);
+    fprintf(out, "%o", (unsigned int) old);
+
+    putc(':', out);
+
+    fprintf(out, "%o) ", (unsigned int) new);
 }
 
 static void show_diffs(integrit_t *it,
@@ -280,7 +298,24 @@ static void show_diffs(integrit_t *it,
 	sho_oct(stdout, 'p', "permissions",
 		 sa->st_mode & PERM_MASK, sb->st_mode & PERM_MASK);
       } else {
-	show_diff_perms(stdout, sa->st_mode, sb->st_mode);
+	show_diff_perms(stdout,
+                        sa->st_mode & PERM_MASK, sb->st_mode & PERM_MASK);
+      }
+    }
+    if (diffs & RULE_TYPE) {
+      if (it->output == OUTPUT_XML) {
+	sho_oct(stdout, 't', "type",
+		 sa->st_mode & S_IFMT, sb->st_mode & S_IFMT);
+      } else {
+	show_diff_type(stdout,
+                       sa->st_mode & S_IFMT, sb->st_mode & S_IFMT);
+      }
+    }
+    if (diffs & RULE_DEVICETYPE) {
+      if (it->output == OUTPUT_XML) {
+	sho_oct(stdout, 'd', "devicetype", sa->st_rdev, sb->st_rdev);
+      } else {
+	show_diff_type(stdout, sa->st_rdev, sb->st_rdev);
       }
     }
     if (diffs & RULE_NLINK)
@@ -317,8 +352,16 @@ static void report_stat_differences(integrit_t *it,
 	&& (sa->st_ino != sb->st_ino))
       diffs	 |= RULE_INODE;
     if ((flags & RULE_PERMS)
-	&& (sa->st_mode != sb->st_mode))
+	&& ((sa->st_mode & PERM_MASK) != (sb->st_mode & PERM_MASK)))
       diffs	 |= RULE_PERMS;
+    if ((flags & RULE_TYPE)
+	&& ((sa->st_mode & S_IFMT) != (sb->st_mode & S_IFMT)))
+      diffs	 |= RULE_TYPE;
+    if ((flags & RULE_DEVICETYPE)
+	&& ((S_ISCHR(sa->st_mode) || S_ISBLK(sa->st_mode))
+	&& ((S_ISCHR(sb->st_mode) || S_ISBLK(sb->st_mode)))
+	&& (sa->st_rdev != sb->st_rdev)))
+      diffs	 |= RULE_DEVICETYPE;
     if ((flags & RULE_NLINK)
 	&& (sa->st_nlink != sb->st_nlink))
       diffs	 |= RULE_NLINK;
@@ -347,28 +390,6 @@ static void report_stat_differences(integrit_t *it,
     }
 }
 
-/* report that a file has changed from a non-regular file
- * to a regular file */
-static void report_2regfile(integrit_t *it, const char *path)
-{
-    it->exit_status |= INTEGRIT_EXIT_CHANGES;
-
-    switch (it->output) {
-      case OUTPUT_XML:
-	XML_CHANGE_START_PRINT(stdout, "filetype", path);
-	XML_ELEMENT_PRINT(stdout, "old", "not regular file");
-	XML_ELEMENT_PRINT(stdout, "new", "regular file");
-	XML_END_PRINT(stdout, "change");
-	putc('\n', stdout);
-	break;
-      case OUTPUT_LINES:
-	printf("changed: %s   became regular file\n", path);
-	break;
-      default:
-	abort();		/* shouldn't happen */
-    }
-}
-
 /* report that the checksum has changed. */
 static void report_sumchange(integrit_t *it, const char *path,
 			     const unsigned char *old, size_t oldsiz,
@@ -416,10 +437,11 @@ static void report_differences(integrit_t *it, fileinfo *currinf)
     if (cdb_get(knowndb, &old) == -1)
       die(__FUNCTION__, "Error: cdb_get entry for file (%s)", path);
 
-    if (S_ISREG(currinf->dbinf.stat.st_mode)) {	/* if it's a regular file */
-      if (knownsiz != sizeof(dbinfo)) /* maybe known has no checksum */
-	report_2regfile(it, path);
-      else if (flags & RULE_SUM) {
+    if (S_ISREG(currinf->dbinf.stat.st_mode) ||
+	S_ISLNK(currinf->dbinf.stat.st_mode)) {
+
+      /* if it's a regular file or symlink */
+      if (flags & RULE_SUM) {
 	if (! currinf->did_sum)
 	  currinf->did_sum	 = do_checksum(it, currinf);
 
@@ -449,13 +471,19 @@ static void show_xml_octal(FILE *out, const char *type, unsigned long val)
 /* similar to utils/show.c: show_entry */
 static void report_newfile(integrit_t *it, fileinfo *inf)
 {
-    char	*path	 = inf->path;
-    struct stat	*s	 = &inf->dbinf.stat;
+    char		*path	 = inf->path;
+    struct stat		*s	 = &inf->dbinf.stat;
+    unsigned long	flags	 = fileinfo_ruleset(it, inf);
+    unsigned char	zerosum[SHA1_DIGEST_LENGTH];
 
     switch (it->output) {
       case OUTPUT_XML:
 	XML_CHANGE_START_PRINT(stdout, "newfile", path);
 	show_xml_octal(stdout, "permissions", s->st_mode & PERM_MASK);
+	show_xml_octal(stdout, "type", s->st_mode & S_IFMT);
+	if (S_ISBLK(s->st_mode) || S_ISCHR(s->st_mode)) {
+	  show_xml_octal(stdout, "devicetype", s->st_rdev);
+	}
 	show_xml_long(stdout, "uid", s->st_uid);
 	show_xml_long(stdout, "gid", s->st_gid);
 	show_xml_long(stdout, "size", s->st_size);
@@ -467,10 +495,11 @@ static void report_newfile(integrit_t *it, fileinfo *inf)
 	 * with "missing:" and "changed:" */
 	printf("new:     %s   ", path); /* this file wasn't in known db */
 
-	if (S_ISLNK(s->st_mode)) /* for a symlink ... */
-	  fputs("p(sym) ", stdout); /* display "sym" instead of "777" */
-	else
-	  show_octal(stdout, 'p', s->st_mode & PERM_MASK);
+	show_octal(stdout, 'p', s->st_mode & PERM_MASK);
+	show_octal(stdout, 't', s->st_mode & S_IFMT);
+	if (S_ISBLK(s->st_mode) || S_ISCHR(s->st_mode)) {
+	  show_octal(stdout, 'd', s->st_rdev);
+        }
 
 	show_long(stdout, 'u', s->st_uid);
 	show_long(stdout, 'g', s->st_gid);
@@ -482,6 +511,18 @@ static void report_newfile(integrit_t *it, fileinfo *inf)
 	break;
     }
     putc('\n', stdout);
+    /* If it's a regular file or symlink, report an SHA-1 change */
+    if (S_ISREG(s->st_mode) || S_ISLNK(s->st_mode)) {
+      if (flags & RULE_SUM) {
+        if (! inf->did_sum)
+          inf->did_sum	 = do_checksum(it, inf);
+        if (inf->did_sum) {
+          memset(zerosum, 0, SHA1_DIGEST_LENGTH);
+          report_sumchange(it, path, zerosum, sizeof(zerosum),
+               inf->dbinf.sum, sizeof(inf->dbinf.sum));
+        }
+      }
+    }
 }    
 
 inline static void do_check(integrit_t *it, fileinfo *inf)
@@ -508,7 +549,8 @@ inline static void do_update(integrit_t *it, fileinfo *inf)
     dbinfo		*dbinf	 = &inf->dbinf;
     size_t		datasiz;
     
-    if (S_ISREG(dbinf->stat.st_mode))
+    if (S_ISREG(dbinf->stat.st_mode) ||
+	S_ISLNK(dbinf->stat.st_mode))
       /* BUGGISH: don't test inf->did_sum, since this function's called
        *          before do_check (very minorly buggish :)
        */
