@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -18,6 +18,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 /* elcerror.c */
-void die(char *function, char *fmt, ...);
-void warn(char *function, char *fmt, ...);
+void die(const char *function, const char *fmt, ...);
+void warn(const char *function, const char *fmt, ...);
 
