@@ -1,6 +1,6 @@
 /*
 hashtbl - a handy and fast, free hash table implementation
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"leakfind.h"	/* checking for memory leaks */
 #endif
 
-void chomp(char *chp)
+void remove_first_newline(char *chp)
 {
     /* remove trailing newlines from a one-line string */
     for( ; *chp; ++chp)
@@ -75,7 +75,7 @@ int main(void)
     while (fgets(buf, BUFSIZ, words)) {
       void *replaced;
       
-      chomp(buf);
+      remove_first_newline(buf);
       intp	 = copyint(i++);
       if (hashtbl_store(&h, buf, strlen(buf), intp, &replaced) == -1) {
 	perror("hashtbl_store");
@@ -89,7 +89,7 @@ int main(void)
 
     rewind(words);
     while (fgets(buf, BUFSIZ, words)) {
-      chomp(buf);
+      remove_first_newline(buf);
       if (! (intp = hashtbl_lookup(&h, buf, strlen(buf))) )
 	puts("not found");
       else
@@ -100,7 +100,7 @@ int main(void)
     
     rewind(words);
     while (fgets(buf, BUFSIZ, words)) {
-      chomp(buf);
+      remove_first_newline(buf);
       if (! (intp = hashtbl_remove(&h, buf, strlen(buf))) ) {
 	puts("not found");
       } else {
@@ -115,7 +115,7 @@ int main(void)
     while (fgets(buf, BUFSIZ, words)) {
       void *replaced;
 
-      chomp(buf);
+      remove_first_newline(buf);
       intp	 = copyint(i++);
       if (hashtbl_store(&h, buf, strlen(buf), intp, &replaced) == -1) {
 	perror("hashtbl_store");
