@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"leakfind.h"
 #endif
 
-void hexprint(FILE *out, const char *buf, size_t n)
+void hexprint(FILE *out, const unsigned char *buf, size_t n)
 {
     while (n--)
       fprintf(out, "%02x", 0xff & *buf++);
