@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001, 2002 Ed L. Cashin
+Copyright (C) 2005 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 void show_long(FILE *out, char T, long val);
 void show_octal(FILE *out, char T, unsigned long val);
 void show_time(FILE *out, char T, time_t val);
-void show_checksum(FILE *out, const char *sum, size_t siz);
+void show_checksum(FILE *out, const unsigned char *sum, size_t siz);
 void show_entry(int show_checksums,
 		const char *key, size_t klen, dbinfo *val, size_t vlen);
 
